# ******************* DEBUG ******************** #
#' output to screen, with a flush.console
#' @param ... fed in to \code{message}
#' @seealso verbosePrintf
#' @note This function calls \code{\link[utils]{flush.console}} after
#' printing out the message, so that (for example) in heavy-processing loops
#' containing per-loop messages, the console is always kept up-to-date (as
#' opposed to spewing out all the messages at the end of the loop).
#' @examples \dontrun{
#' # demonstrating the 'flush.console()': try:
#' for (i in 1:10) {
#'     message(i)
#'     A <- runif(10000000);
#' }
#' # Notice how the messages all get output all at once at the end of the loop.
#' # (You may need to play with the number in `runif`)
#' # This is due to the `runif` being processor-heavy.
#' # Now try:
#' for (i in 1:10) {
#'     verbosePrint(i)
#'     # Like saying message(i); flush.console()
#'     A <- runif(10000000);
#' }
#' # Note how the messages appear in real-time now.
#' }
#' @family debug
#' @export
verbosePrint <- function(...) {
    message(...)
    flush.console()
}

#' output to screen, like sprintf.
#' @param ... fed in to \code{sprintf} which is then \code{message}
#' @seealso \code{\link[base]{sprintf}} for formatting instructions.
#' @family debug
#' @export printf verbosePrintf
#' @aliases printf verbosePrintf
printf <- verbosePrintf <- function(...) {
    message(sprintf(...))
    flush.console()
}

#' throw a formatted warning.
#' Essentially calls \code{warning(sprintf(...))}
#' @param ... arguments to sprintf
#' @param call. logical, indicating if call should become part of the
#' warning message (see \code{\link[base]{warning}})
#' @param immediate. logical, indicating of the call should be output
#' immediately, even if \code{getOption("warn")<=0}
#' @param domain see \code{\link[base]{warning}}
#' @note See \code{\link[base]{warning}} for explanation of parameters.
#' @family debug
#' @seealso \code{\link[base]{warning}}, \code{\link[base]{sprintf}}
#' @export
warningf <- function(..., call.=TRUE, immediate.=FALSE, domain=NULL) {
    # raise the warning up to the parent function, not the warningf function
    if (call.) {
        warning(paste('In', sys.call(-1), ':', sprintf(...)), call.=FALSE,
                immediate.=immediate., domain=domain)
    } else {
        warning(sprintf(...), call.=call., immediate.=immediate., domain=domain)
    }
}

#' throw a formated error
#' Essentially calls \code{stop(sprintf(...))}
#' @param ... arguments to sprintf
#' @param call. logical, indicating if call should become part of the
#' error message (see \code{\link[base]{stop}})
#' @param domain see \code{\link[base]{stop}}
#' See \code{\link[base]{stop}} for the explanation of the other parameters.
#' @family debug
#' @seealso \code{\link[base]{stop}}, \code{\link[base]{sprintf}}
#' @export
stopf <- function(..., call.=TRUE, domain=NULL) {
    # raise the error up to the parent function, not the stopf function.
    if (call.) {
        stop(paste('In', sys.call(-1), ':', sprintf(...)), call.=FALSE,
             domain=domain)
    } else {
        stop(sprintf(...), call.=call., domain=domain)
    }
}


#' Catch *and* save both errors and warnings, and in the case of
#' a warning, also keep the computed result.
#'
#' @title tryCatch both warnings and errors
#' @param expr the expression to evaluate
#' @return a list with 'value' and 'warning', where
#'  'value' may be an error caught.
#' @author Martin Maechler
#' @family debug
#' @export
tryCatch.W.E <- function(expr) {
    W <- NULL
    w.handler <- function(w) { # warning handler
        W <<- w
        invokeRestart("muffleWarning")
    }
    list(value=withCallingHandlers(tryCatch(expr, error=function(e) e),
                                     warning=w.handler),
         warning=W)

}

#' convert a delimited string to a vector.
#' @param x the delimited string.
#' @details Essentially calls c(...) on the arguments, after
#'         delimiting them.
#'         Delimited by 'x' or ', '
#' @examples
#' evaluateSplit('1, 2, 3')       # returns c(1, 2, 3)
#' evaluateSplit('640x480') # returns c(640, 480)
#' @family demo
#' @family evalParse
# NOTE:
# gWidgetstcltk has this problem where if you do
# svalue(obj) AND if you created obj with a coerce.with, it will
#  call coerce.with TWICE:
# it does  coerce.with(coerce.with(string_value_of(obj))).
# Hence the reason for the curious paste(x, collapse=', ').
#' @export
evaluateSplit <- function(x) {
    evaluate(sprintf('c(%s)', gsub('x', ', ', paste(x, collapse=', '))))
}

#' evaluate string as an R expression
#' @param x string with command to be evaluated
#' @param envir the environment to evaluate `x`. By default, `parent.frame()`.
#' @return result of evaluating x as if it were an R expression
#' @family demo
#' @family evalParse
#' @seealso \code{\link[base:sys.parent]{parent.frame}}
#' @seealso \code{\link[base]{eval}}
#' @export
evaluate <- function(x, envir=parent.frame()) {
    eval(parse(text=x), envir)
}

# ********************* DEMOS ********************** #
# (these functions are really dumb...should I just deprecate them?
#  or at least thing of better names!)
#' *P*rints a string and then *Ev*aluates it
#'
#' Useful for interactive demos.
#' @param str the command to evaluate
#' @return the result of the expression
#' @family demo
#' @export
PEv <- function(str) {
    message(str, "\n\n");
    evaluate(str, parent.frame())
}

#' Pauses evaluation, waiting for user input.
#'
#' This displays "press ENTER to continue" and waits until the user has done so.
#' Useful for interactive demos.
#' @family demo
#' @export
wait <- function() {
    message("                                                              press ENTER to continue\n");
    scan(file="", quiet=TRUE, what=character(0), nlines=1)
}

#' *D*isplays a command, *W*aits, and *E*xecutes.
#'
#' This displays a command, waits for the user to press enter,
#'  and then executes the command.
#' Useful for interactive demos.
#' @param str the command to evaluate
#' @return the result of the expression
#' @family demo
#' @export
DWE <- function(str) {
    message(str, "\n\n")
    message("                                                              press ENTER to evaluate\n");
    scan(file="", quiet=TRUE, what=character(0), nlines=1)
    eval(parse(text=str), parent.frame())
}
