# NOTE: this file depends on 'test-includes.r', sourced in utilitiesR/tests/test-all.r
# source('test-includes.r')
context("Debugging and printing")

describe('verbosePrint', {
    test("exists and is a function", {
        expect_that('verbosePrint', is_exported('function'))
    })

    test("prints to the console", {
        expect_that(verbosePrint('blahblahblah', 'haha'),
                    shows_message('blahblahblahhaha'))
    })

    test("can be muffled with suppressMessages", {
        expect_that(suppressMessages(verbosePrint('haha')),
                    not(shows_message()))
    })
})

describe('printf', {
    test("exists and is a function", {
        expect_that('printf', is_exported('function'))
    })

    test("prints to the console", {
        expect_that(printf('blahblahblah'),
                    shows_message('blahblahblah'))
    })

    test("treats arguments like `sprintf`", {
        expect_that(printf('abc %s %i', 'def', 1),
                    shows_message('abc def 1'))
    })

    test("can be muffled with suppressMessages", {
        expect_that(suppressMessages(printf('haha %s', 'you')),
                    not(shows_message()))
    })
    
    test("has synonym verbosePrintf", {
        expect_that(printf, is_identical_to(verbosePrintf))
    })
})

describe('warningf', {
    test("exists and is a function", {
        expect_that('warningf', is_exported('function'))
    })

    test("gives a warning, treating arguments as if through `sprintf`", {
        expect_that(warningf("I do not like green eggs and ham"),
                    gives_warning())
        expect_that(warningf("I do not like green eggs and ham"),
                    gives_warning("I do not like green eggs and ham"))
        expect_that(warningf("I do not like %s and %s", "green eggs", "ham"),
                    gives_warning("I do not like green eggs and ham"))
    })

    test("fires the warning as coming from the parent function, not from warningf", {
        f <- function () { warningf("I do not like them, Sam I Am!") }
        expect_that(f(),
                    gives_warning('In f : I do not like them, Sam I Am!'))
    })

    test("can be muffled with suppressWarnings", {
        expect_that(suppressWarnings(warningf('lkjaskljdf')),
                    not(gives_warning()))
    })
})

describe('stopf', {
    test("exists and is a function", {
        expect_that('stopf', is_exported('function'))
    })

    test("throws an error, treating arguments as if through `sprintf`", {
        expect_that(stopf("I do not like green eggs and ham"),
                    throws_error())
        expect_that(stopf("I do not like green eggs and ham"),
                    throws_error("I do not like green eggs and ham"))
        expect_that(stopf("I do not like %s and %s", "green eggs", "ham"),
                    throws_error("I do not like green eggs and ham"))
    })

    test("fires the error as coming from the parent function, not from stopf", {
        f <- function () { stopf("I do not like them, Sam I Am!") }
        expect_that(f(),
                    throws_error('In f : I do not like them, Sam I Am!'))
    })
})

# TODO: deprecate: unsure what exactly this is *meant* to do, seems to only
# store the last warning or error?
describe('tryCatch.W.E', {
    test("exists and is a function", {
        expect_that('tryCatch.W.E', is_exported('function'))
    })

    test("returns the last result calculated", {
        out <- tryCatch.W.E({log(2)})
        expect_that(out, is_a('list'))
        expect_that(names(out), is_identical_to(c('value', 'warning')))
        expect_that(out$value, equals(log(2)))
        expect_that(out$warning, is_null())
    })

    test("captures warnings, returning the last calculated result in out$value and the warning in out$warning", {
        out <- tryCatch.W.E({log(-1); 2})
        expect_that(out, is_a('list'))
        expect_that(names(out), is_identical_to(c('value', 'warning')))
        expect_that(out$value, equals(2))
        expect_that(out$warning, is_a('warning'))
        expect_that(gettext(out$warning), matches('NaNs produced'))
    })

    test("captures errors, returning the error in out$value and any warnings in out$warning", {
        out <- tryCatch.W.E({log(-1); stop('boo!')})
        expect_that(out, is_a('list'))
        expect_that(names(out), is_identical_to(c('value', 'warning')))

        expect_that(out$value, is_a('error'))
        expect_that(out$warning, is_a('warning'))
        expect_that(geterrmessage(), is_identical_to('boo!'))
        expect_that(gettext(out$warning), matches('NaNs produced'))
    })
})

describe('evaluateSplit', {
    test("exists and is a function", {
        expect_that('evaluateSplit', is_exported('function'))
    })

    test("converts 'x' or ','-delimited strings into vectors", {
        expect_that(evaluateSplit('1, 2,3'), equals(1:3))
        expect_that(evaluateSplit('1x2 x3 x 4x5'), equals(1:5))
    })
})

describe('evaluate', {
    test("exists and is a function", {
        expect_that('evaluate', is_exported('function'))
    })

    test("evaluates the input string", {
        suppressWarnings(rm(x))
        evaluate('x <- 1')
        expect_that(x, equals(1))
    })
})

# should I deprecate these ? they are a little bit silly...
describe('PEv', {
    test("exists and is a function", {
        expect_that('PEv', is_exported('function'))
    })

    test("prints to the console and then evaluates", {
        suppressWarnings(rm(x))
        expect_that(PEv('x <- 1'),
                    shows_message('x <- 1'))
        expect_that(x, equals(1))
    })

    test("can be muffled with suppressMessages", {
        expect_that(suppressMessages(PEv('x <- 1')),
                    not(shows_message()))
    })
})

# TODO: how to test that we wait for the user to press ENTER?
describe('wait', {
    test("exists and is a function", {
        expect_that('wait', is_exported('function'))
    })
})

describe('DWE', {
    test("exists and is a function", {
        expect_that('DWE', is_exported('function'))
    })
})
